@@ -2,7 +2,9 @@
 Serializers for Calculations module
 """
 import json
+import uuid
 from tokenize import TokenError
+from datetime import date
 
 import pint
 from rest_framework import serializers
@@ -11,6 +13,7 @@ from sympy import sympify, SympifyError
 from djangophysics.units.models import UnitSystem
 from .models import Expression, CalculationPayload, Operand, \
     CalculationResult, CalculationResultError, CalculationResultDetail
+from djangophysics.core.helpers import uuid4_str
 
 
 class OperandSerializer(serializers.Serializer):
@@ -86,6 +89,11 @@ class ExpressionSerializer(serializers.Serializer):
     """
     unit_system = None
     key = None
+    exp_id = serializers.CharField(default=uuid4_str)
+    value_date = serializers.DateField(
+        label="Date of value for currency conversion",
+        required=False
+    )
     expression = serializers.CharField(
         label="Mathematical expression to evaluate",
         required=True,
@@ -193,6 +201,9 @@ class ExpressionSerializer(serializers.Serializer):
         except TokenError:
             self._errors['expression'] = "Invalid expression"
             return False
+        except ZeroDivisionError:
+            self._errors['expression'] = "Division by zero"
+            return False
         if out_units:
             try:
                 result.to(out_units)
@@ -229,6 +240,9 @@ class ExpressionSerializer(serializers.Serializer):
         except TokenError:
             self._errors['expression'] = "Invalid expression"
             return False
+        except ZeroDivisionError:
+            self._errors['expression'] = "Division by zero"
+            return False
         if out_units:
             try:
                 result.to(out_units)
@@ -274,6 +288,9 @@ class ExpressionSerializer(serializers.Serializer):
         except pint.errors.DimensionalityError as e:
             self._errors['out_units'] = f"Incoherent output dimension {str(e)}"
             return None
+        except pint.UndefinedUnitError as e:
+            self._errors['out_units'] = f"Incoherent output units {str(e)}"
+            return None
         return out_units
 
     def operands_validation(self, operands):
@@ -307,6 +324,8 @@ class ExpressionSerializer(serializers.Serializer):
             if vs.is_valid():
                 operands.append(vs.create(vs.validated_data))
         return Expression(
+            exp_id=validated_data.get('exp_id', uuid.uuid4()),
+            value_date=validated_data.get('value_date', date.today()),
             expression=validated_data.get('expression'),
             operands=operands,
             out_units=validated_data.get('out_units')
@@ -318,6 +337,8 @@ class ExpressionSerializer(serializers.Serializer):
         :param instance: Expression object
         :param validated_data: cleaned data
         """
+        instance.exp_id = validated_data.get('exp_id')
+        instance.value_date = validated_data.get('value_date')
         instance.expression = validated_data.get('expression')
         instance.operands = validated_data.get('operands')
         instance.out_units = validated_data.get('out_units')
@@ -328,6 +349,11 @@ class CalculationResultDetailSerializer(serializers.Serializer):
     """
     Serializer for the CalculationResultDetail class
     """
+    exp_id = serializers.CharField(label="ID of the expression")
+    value_date = serializers.DateField(
+        label="Date of value for currency conversion",
+        required=False
+    )
     expression = serializers.CharField(label="Expression to evaluate")
     operands = OperandSerializer(many=True)
     magnitude = serializers.FloatField(label="Magnitude of result")
@@ -347,6 +373,10 @@ class CalculationResultDetailSerializer(serializers.Serializer):
         :param instance: CalculationResultDetail object
         :param validated_data: cleaned data
         """
+        instance.exp_id = validated_data.get(
+            'exp_id', instance.exp_id)
+        instance.value_date = validated_data.get(
+            'value_date', instance.value_date)
         instance.expression = validated_data.get(
             'expression', instance.expression)
         instance.operands = validated_data.get(
@@ -358,14 +388,29 @@ class CalculationResultDetailSerializer(serializers.Serializer):
         return instance
 
 
+class CalculationResultErrorDetailSerializer(serializers.Serializer):
+    """
+    List of errors in an expression or calculation
+    """
+    source = serializers.CharField(label="Source of the error")
+    error = serializers.CharField(label="Detail of the error")
+
+
 class CalculationResultErrorSerializer(serializers.Serializer):
     """
     Serializer for the CalculationResultError class
     """
+    exp_id =  serializers.CharField(label="ID of the expression")
+    value_date = serializers.DateField(
+        label="Date of value for currency conversion",
+        required=False
+    )
     expression = serializers.CharField(label="Expression to evaluate")
     operands = OperandSerializer(many=True)
     calc_date = serializers.DateField(label="Date of calculation")
-    error = serializers.CharField(label="Error during calculation")
+    errors = CalculationResultErrorDetailSerializer(
+        label="Error during calculation",
+        many=True)
 
     def create(self, validated_data):
         """
@@ -380,14 +425,16 @@ class CalculationResultErrorSerializer(serializers.Serializer):
         :param instance: CalculationResultError object
         :param validated_data: cleaned data
         """
+        instance.exp_id = validated_data.get(
+            'exp_id', instance.exp_id)
         instance.expression = validated_data.get(
             'expression', instance.expression)
         instance.operands = validated_data.get(
             'operands', instance.operands)
         instance.calc_date = validated_data.get(
             'calc_date', instance.calc_date)
-        instance.error = validated_data.get(
-            'error', instance.error)
+        instance.errors = validated_data.get(
+            'errors', instance.errors)
         return instance
 
 
@@ -414,6 +461,7 @@ class CalculationResultSerializer(serializers.Serializer):
         :param validated_data: cleaned data
         """
         instance.id = validated_data.get('id', instance.id)
+        instance.value_date = validated_data.get('value_date', instance.value_date)
         instance.detail = validated_data.get('detail', instance.detail)
         instance.status = validated_data.get('status', instance.status)
         instance.errors = validated_data.get('errors', instance.errors)
@@ -479,6 +527,7 @@ class CalculationPayloadSerializer(serializers.Serializer):
         :param validated_data: cleaned data
         """
         self.data = validated_data.get('data', instance.data)
+        self.value_date = validated_data.get('value_date', instance.value_date)
         self.unit_system = validated_data.get(
             'unit_system', instance.unit_system)
         self.batch_id = validated_data.get('batch_id', instance.batch_id)
