@@ -1,8 +1,11 @@
 """
 Djangophysics GraphQL schemas
 """
+from datetime import datetime, date
+
 from ariadne import QueryType, gql, make_executable_schema
 
+from ..core.helpers import validate_language, service
 from ..countries.models import Country, CountrySubdivision
 from ..currencies.models import Currency
 from ..rates.models import Rate
@@ -60,6 +63,44 @@ type_defs = '''
         current_time: String!
     }
     
+    """
+    GPS Location
+    """
+    type Location    {
+        "Latitude"
+        lat: Float!,
+        "Longitude"
+        lng: Float!
+    }
+    
+    """
+    Address  
+    """
+    type Address {
+        "GPS coordinates"
+        location: Location,
+        "Street number"
+        street_number: String,
+        "Street name"
+        street: String,
+        "Postal code"
+        postal_code: String,
+        "Locality or city name"
+        locality: String
+        "County or administrative level"
+        county: CountrySubdivision,
+        "County or administrative level name"
+        county_label: String,
+        "Subdivision name"
+        subdivision_label: String,
+        "Subdivision object"
+        subdivision: CountrySubdivision,
+        "Country object"
+        country: Country
+        "Result confidence, not always available"
+        confidence: Int,
+    }
+    
     """
     List of countries based on the ISO-3166
     """
@@ -197,6 +238,10 @@ type_defs = '''
         countries(term: String): [Country]
         "Country details"
         country(alpha_2: String!): Country
+        "Geocode interface: key is the API key to the geocoding service"
+        geocode(address: String!, key: String, language: String, geocoder: String): [Address]
+        "Reverse Geocode interface: key is the API key to the geocoding service"
+        reverse_geocode(atitude: Float!, longitude: Float!, key: String, language: String, geocoder: String): [Address]
         "Searchable list of subdivisions for a country"
         country_subdivisions_search(search_term: String!, country_code: String, ordering: String): [CountrySubdivision]
         "List of subdivisions for a country"
@@ -341,10 +386,11 @@ def resolve_rate(_, info,
     :param currency: currency  ISO4217 code
     :param value_date: date of value for the rate "YYYY-MM-DD"
     """
+    date_obj = datetime.strptime(value_date, '%Y-%m-%d').date()
     rate = Rate.objects.find_rate(
         base_currency=base_currency.upper(),
         currency=currency.upper(),
-        date_obj=value_date
+        date_obj=date_obj
     )
     return rate
 
@@ -374,7 +420,8 @@ def resolve_rates_page(_, info,
     if base_currency:
         rates = rates.filter(base_currency=base_currency)
     if value_date:
-        rates = rates.filter(value_date=value_date)
+        date_obj = datetime.strptime(value_date, '%Y-%m-%d').date()
+        rates = rates.filter(value_date=date_obj)
     cnt = rates.count()
     rates = rates[page * page_size: (page + 1) * page_size]
     return {
@@ -417,4 +464,37 @@ def resolve_dimension(_, info, system_name, unit_name):
     return us.unit(unit_name=unit_name)
 
 
+@query.field("geocode")
+def resolve_geocode(_, info, address: str, key: str,
+                    language: str = 'en', geocoder: str = 'google'):
+    language = validate_language(lang=language)
+    geocoder = service(
+        service_type='geocoding',
+        service_name=geocoder
+    )
+    data = geocoder.search(
+        address=address,
+        key=key,
+        language=language
+    )
+    return geocoder.addresses(data)
+
+
+@query.field("reverse_geocode")
+def resolve_reverse_geocode(_, info, latitude: float, longitude: float, key: str,
+                    language: str = 'en', geocoder: str = 'google'):
+    language = validate_language(lang=language)
+    geocoder = service(
+        service_type='geocoding',
+        service_name=geocoder
+    )
+    data = geocoder.reverse(
+        lat=latitude,
+        lng=longitude,
+        key=key,
+        language=language
+    )
+    return geocoder.addresses(data)
+
+
 schema = make_executable_schema(type_defs, query)
